window.Gleam.initSearch([{"doc":"glitr_convert_cake","title":"glitr_convert_cake","content":"# glitr_convert_cake\n\n[![Package Version](https://img.shields.io/hexpm/v/glitr_convert_cake)](https://hex.pm/packages/glitr_convert_cake)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glitr_convert_cake/)\n\n**Easily select, insert, update and decode values from a database using cake and glitr_convert** \n\n## Installation\n\n```sh\ngleam add glitr_convert_cake\n```\n\n## Usage\n\n```gleam\nimport cake\nimport cake/dialect/postgres_dialect\nimport cake/insert as i\nimport cake/param\nimport cake/select as s\nimport cake/update as u\nimport cake/where as w\nimport gleam/io\nimport gleam/pgo\nimport gleam/result\nimport glitr/convert as c\nimport glitr/convert/cake as cc\n\npub type User {\n  User(id: String, name: String, age: Int)\n}\n\npub fn user_converter() -&gt; c.Converter(User) {\n  c.object({\n    use id &lt;- c.parameter\n    use name &lt;- c.parameter\n    use age &lt;- c.parameter\n    use &lt;- c.constructor\n    User(id:, name:, age:)\n  })\n  |&gt; c.field(&quot;id&quot;, fn(v) { Ok(v.id) }, c.string())\n  |&gt; c.field(&quot;name&quot;, fn(v) { Ok(v.name) }, c.string())\n  |&gt; c.field(&quot;age&quot;, fn(v) { Ok(v.age) }, c.int())\n  |&gt; c.to_converter\n}\n\npub fn main() {\n  let alice = User(&quot;1&quot;, &quot;Alice&quot;, 21)\n  let bob = User(&quot;2&quot;, &quot;Bob&quot;, 22)\n  let table_name = &quot;users&quot;\n\n  let create_query = i.new()\n    |&gt; i.table(table_name)\n    |&gt; cc.cake_insert(user_converter(), [alice, bob])\n    |&gt; i.to_query\n    // Execute query\n\n  let update_query = u.new()\n    |&gt; u.table(table_name)\n    |&gt; cc.cake_update(user_converter(), User(..alice, age: 36))\n    |&gt; u.where(w.col(&quot;id&quot;) |&gt; w.eq(w.string(alice.id)))\n    |&gt; u.to_query\n    // Execute query\n\n  let select_all_query = s.new()\n    |&gt; s.from_table(table_name)\n    |&gt; cc.cake_select_fields(user_converter())\n    |&gt; s.to_query\n\n  // Example: executing query with pgo and decoding the values with glitr_convert_cake\n  let query = select_all_query |&gt; postgres_dialect.read_query_to_prepared_statement\n  \n  pgo.execute(\n    query |&gt; cake.get_sql,\n    db_connection, // Connection to a Postgres DB\n    query |&gt; cake.get_params |&gt; list.map(map_param),\n    cc.cake_decode(user_converter())\n  )\n  |&gt; result.map(fn (res) { res.rows })\n  |&gt; io.debug\n  // Ok(List(User(&quot;1&quot;, &quot;Alice&quot;, 36), User(&quot;2&quot;, &quot;Bob&quot;, 22)))\n}\n\nfn map_param(p: param.Param) -&gt; pgo.Value {\n  case p {\n    param.BoolParam(v) -&gt; pgo.bool(v)\n    param.FloatParam(v) -&gt; pgo.float(v)\n    param.IntParam(v) -&gt; pgo.int(v)\n    param.StringParam(v) -&gt; pgo.text(v)\n    param.NullParam -&gt; pgo.null()\n  }\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/glitr_convert_cake&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"glitr/convert/cake","title":"cake_decode","content":"pub fn cake_decode(\n  converter: Converter(a),\n) -&gt; fn(Dynamic) -&gt; Result(a, List(DecodeError))\n Decode a Dynamic value from a database row using the provided converter.\nSynonyms:\nglitr/convert/cake.cake_decode\nglitr/convert/cake cake_decode","url":"glitr/convert/cake.html#cake_decode"},{"doc":"glitr/convert/cake","title":"cake_insert","content":"pub fn cake_insert(\n  value: Insert(a),\n  converter: Converter(a),\n  values: List(a),\n) -&gt; Insert(a)\n Sets the columns and values of the Insert using the converter and the provided values.  \n If the converter isn&#39;t an Object converter, the value is left unchanged.  \n This is because we don&#39;t have information about the fields/columns names otherwise.\nSynonyms:\nglitr/convert/cake.cake_insert\nglitr/convert/cake cake_insert","url":"glitr/convert/cake.html#cake_insert"},{"doc":"glitr/convert/cake","title":"cake_select_fields","content":"pub fn cake_select_fields(\n  value: Select,\n  converter: Converter(a),\n) -&gt; Select\n Sets the selected colums of the Select using the converter.  \n If the converter isn&#39;t an Object converter, the value is left unchanged.\n This is because we don&#39;t have information about the fields/columns names otherwise.\nSynonyms:\nglitr/convert/cake.cake_select_fields\nglitr/convert/cake cake_select_fields","url":"glitr/convert/cake.html#cake_select_fields"},{"doc":"glitr/convert/cake","title":"cake_update","content":"pub fn cake_update(\n  value: Update(a),\n  converter: Converter(a),\n  update: a,\n) -&gt; Update(a)\n Sets the column sets of the Update using the converter and the provided value.  \n If the converter isn&#39;t an Object converter, the value is left unchanged.  \n This is because we don&#39;t have information about the fields/columns names otherwise.\nSynonyms:\nglitr/convert/cake.cake_update\nglitr/convert/cake cake_update","url":"glitr/convert/cake.html#cake_update"},{"doc":"glitr/convert/cake","title":"glitr/convert/cake","content":"","url":"glitr/convert/cake.html"},{"doc":"glitr_convert_cake","title":"main","content":"pub fn main() -&gt; Nil\n\nSynonyms:\nglitr_convert_cake.main\nglitr_convert_cake main","url":"glitr_convert_cake.html#main"},{"doc":"glitr_convert_cake","title":"glitr_convert_cake","content":"","url":"glitr_convert_cake.html"}]);